@@ -10,7 +10,7 @@
 use crate::ir::condcodes::{FloatCC, IntCC};
 use crate::ir::types::*;
 use crate::ir::Inst as IRInst;
-use crate::ir::{Opcode, Type};
+use crate::ir::{Opcode, Type, Value};
 use crate::machinst::lower::*;
 use crate::machinst::*;
 use crate::{CodegenError, CodegenResult};
@@ -25,6 +25,8 @@ use regalloc::{Reg, Writable};
 use smallvec::SmallVec;
 use std::cmp;
 
+pub mod isle;
+
 //============================================================================
 // Result enum types.
 //
@@ -270,20 +272,20 @@ fn extend_reg<C: LowerCtx<I = Inst>>(
 }
 
 /// Lowers an instruction input to multiple regs
-fn lower_input_to_regs<C: LowerCtx<I = Inst>>(
+fn lower_value_to_regs<C: LowerCtx<I = Inst>>(
     ctx: &mut C,
-    input: InsnInput,
+    value: Value,
 ) -> (ValueRegs<Reg>, Type, bool) {
-    log::trace!("lower_input_to_regs: input {:?}", input);
-    let ty = ctx.input_ty(input.insn, input.input);
-    let inputs = ctx.get_input_as_source_or_const(input.insn, input.input);
+    log::trace!("lower_value_to_regs: value {:?}", value);
+    let ty = ctx.value_ty(value);
+    let inputs = ctx.get_value_as_source_or_const(value);
     let is_const = inputs.constant.is_some();
 
     let in_regs = if let Some(c) = inputs.constant {
         // Generate constants fresh at each use to minimize long-range register pressure.
         generate_constant(ctx, ty, c as u128)
     } else {
-        ctx.put_input_in_regs(input.insn, input.input)
+        ctx.put_value_in_regs(value)
     };
 
     (in_regs, ty, is_const)
@@ -299,7 +301,17 @@ pub(crate) fn put_input_in_reg<C: LowerCtx<I = Inst>>(
     input: InsnInput,
     narrow_mode: NarrowValueMode,
 ) -> Reg {
-    let (in_regs, ty, is_const) = lower_input_to_regs(ctx, input);
+    let value = ctx.input_as_value(input.insn, input.input);
+    put_value_in_reg(ctx, value, narrow_mode)
+}
+
+/// Like above, only for values
+fn put_value_in_reg<C: LowerCtx<I = Inst>>(
+    ctx: &mut C,
+    value: Value,
+    narrow_mode: NarrowValueMode,
+) -> Reg {
+    let (in_regs, ty, is_const) = lower_value_to_regs(ctx, value);
     let reg = in_regs
         .only_reg()
         .expect("Multi-register value not expected");
@@ -312,7 +324,8 @@ pub(crate) fn put_input_in_regs<C: LowerCtx<I = Inst>>(
     ctx: &mut C,
     input: InsnInput,
 ) -> ValueRegs<Reg> {
-    let (in_regs, _, _) = lower_input_to_regs(ctx, input);
+    let value = ctx.input_as_value(input.insn, input.input);
+    let (in_regs, _, _) = lower_value_to_regs(ctx, value);
     in_regs
 }
 
@@ -365,89 +378,94 @@ fn put_input_in_rse<C: LowerCtx<I = Inst>>(
     input: InsnInput,
     narrow_mode: NarrowValueMode,
 ) -> ResultRSE {
-    let inputs = ctx.get_input_as_source_or_const(input.insn, input.input);
-    if let Some((insn, 0)) = inputs.inst {
-        let op = ctx.data(insn).opcode();
-        let out_ty = ctx.output_ty(insn, 0);
-        let out_bits = ty_bits(out_ty);
-
-        // Is this a zero-extend or sign-extend and can we handle that with a register-mode operator?
-        if op == Opcode::Uextend || op == Opcode::Sextend {
-            let sign_extend = op == Opcode::Sextend;
-            let inner_ty = ctx.input_ty(insn, 0);
-            let inner_bits = ty_bits(inner_ty);
-            assert!(inner_bits < out_bits);
-            if match (sign_extend, narrow_mode) {
-                // A single zero-extend or sign-extend is equal to itself.
-                (_, NarrowValueMode::None) => true,
-                // Two zero-extends or sign-extends in a row is equal to a single zero-extend or sign-extend.
-                (false, NarrowValueMode::ZeroExtend32) | (false, NarrowValueMode::ZeroExtend64) => {
-                    true
-                }
-                (true, NarrowValueMode::SignExtend32) | (true, NarrowValueMode::SignExtend64) => {
-                    true
-                }
-                // A zero-extend and a sign-extend in a row is not equal to a single zero-extend or sign-extend
-                (false, NarrowValueMode::SignExtend32) | (false, NarrowValueMode::SignExtend64) => {
-                    false
-                }
-                (true, NarrowValueMode::ZeroExtend32) | (true, NarrowValueMode::ZeroExtend64) => {
-                    false
-                }
-            } {
-                let extendop = match (sign_extend, inner_bits) {
-                    (true, 8) => ExtendOp::SXTB,
-                    (false, 8) => ExtendOp::UXTB,
-                    (true, 16) => ExtendOp::SXTH,
-                    (false, 16) => ExtendOp::UXTH,
-                    (true, 32) => ExtendOp::SXTW,
-                    (false, 32) => ExtendOp::UXTW,
-                    _ => unreachable!(),
-                };
-                let reg =
-                    put_input_in_reg(ctx, InsnInput { insn, input: 0 }, NarrowValueMode::None);
-                return ResultRSE::RegExtend(reg, extendop);
-            }
-        }
-
-        // If `out_ty` is smaller than 32 bits and we need to zero- or sign-extend,
-        // then get the result into a register and return an Extend-mode operand on
-        // that register.
-        if narrow_mode != NarrowValueMode::None
-            && ((narrow_mode.is_32bit() && out_bits < 32)
-                || (!narrow_mode.is_32bit() && out_bits < 64))
-        {
-            let reg = put_input_in_reg(ctx, input, NarrowValueMode::None);
-            let extendop = match (narrow_mode, out_bits) {
-                (NarrowValueMode::SignExtend32, 1) | (NarrowValueMode::SignExtend64, 1) => {
-                    ExtendOp::SXTB
-                }
-                (NarrowValueMode::ZeroExtend32, 1) | (NarrowValueMode::ZeroExtend64, 1) => {
-                    ExtendOp::UXTB
-                }
-                (NarrowValueMode::SignExtend32, 8) | (NarrowValueMode::SignExtend64, 8) => {
-                    ExtendOp::SXTB
-                }
-                (NarrowValueMode::ZeroExtend32, 8) | (NarrowValueMode::ZeroExtend64, 8) => {
-                    ExtendOp::UXTB
-                }
-                (NarrowValueMode::SignExtend32, 16) | (NarrowValueMode::SignExtend64, 16) => {
-                    ExtendOp::SXTH
-                }
-                (NarrowValueMode::ZeroExtend32, 16) | (NarrowValueMode::ZeroExtend64, 16) => {
-                    ExtendOp::UXTH
-                }
-                (NarrowValueMode::SignExtend64, 32) => ExtendOp::SXTW,
-                (NarrowValueMode::ZeroExtend64, 32) => ExtendOp::UXTW,
-                _ => unreachable!(),
-            };
-            return ResultRSE::RegExtend(reg, extendop);
-        }
+    let value = ctx.input_as_value(input.insn, input.input);
+    if let Some((val, extendop)) = get_as_extended_value(ctx, value, narrow_mode) {
+        let reg = put_value_in_reg(ctx, val, NarrowValueMode::None);
+        return ResultRSE::RegExtend(reg, extendop);
     }
 
     ResultRSE::from_rs(put_input_in_rs(ctx, input, narrow_mode))
 }
 
+fn get_as_extended_value<C: LowerCtx<I = Inst>>(
+    ctx: &mut C,
+    val: Value,
+    narrow_mode: NarrowValueMode,
+) -> Option<(Value, ExtendOp)> {
+    let inputs = ctx.get_value_as_source_or_const(val);
+    let (insn, n) = inputs.inst?;
+    if n != 0 {
+        return None;
+    }
+    let op = ctx.data(insn).opcode();
+    let out_ty = ctx.output_ty(insn, 0);
+    let out_bits = ty_bits(out_ty);
+
+    // Is this a zero-extend or sign-extend and can we handle that with a register-mode operator?
+    if op == Opcode::Uextend || op == Opcode::Sextend {
+        let sign_extend = op == Opcode::Sextend;
+        let inner_ty = ctx.input_ty(insn, 0);
+        let inner_bits = ty_bits(inner_ty);
+        assert!(inner_bits < out_bits);
+        if match (sign_extend, narrow_mode) {
+            // A single zero-extend or sign-extend is equal to itself.
+            (_, NarrowValueMode::None) => true,
+            // Two zero-extends or sign-extends in a row is equal to a single zero-extend or sign-extend.
+            (false, NarrowValueMode::ZeroExtend32) | (false, NarrowValueMode::ZeroExtend64) => true,
+            (true, NarrowValueMode::SignExtend32) | (true, NarrowValueMode::SignExtend64) => true,
+            // A zero-extend and a sign-extend in a row is not equal to a single zero-extend or sign-extend
+            (false, NarrowValueMode::SignExtend32) | (false, NarrowValueMode::SignExtend64) => {
+                false
+            }
+            (true, NarrowValueMode::ZeroExtend32) | (true, NarrowValueMode::ZeroExtend64) => false,
+        } {
+            let extendop = match (sign_extend, inner_bits) {
+                (true, 8) => ExtendOp::SXTB,
+                (false, 8) => ExtendOp::UXTB,
+                (true, 16) => ExtendOp::SXTH,
+                (false, 16) => ExtendOp::UXTH,
+                (true, 32) => ExtendOp::SXTW,
+                (false, 32) => ExtendOp::UXTW,
+                _ => unreachable!(),
+            };
+            return Some((ctx.input_as_value(insn, 0), extendop));
+        }
+    }
+
+    // If `out_ty` is smaller than 32 bits and we need to zero- or sign-extend,
+    // then get the result into a register and return an Extend-mode operand on
+    // that register.
+    if narrow_mode != NarrowValueMode::None
+        && ((narrow_mode.is_32bit() && out_bits < 32) || (!narrow_mode.is_32bit() && out_bits < 64))
+    {
+        let extendop = match (narrow_mode, out_bits) {
+            (NarrowValueMode::SignExtend32, 1) | (NarrowValueMode::SignExtend64, 1) => {
+                ExtendOp::SXTB
+            }
+            (NarrowValueMode::ZeroExtend32, 1) | (NarrowValueMode::ZeroExtend64, 1) => {
+                ExtendOp::UXTB
+            }
+            (NarrowValueMode::SignExtend32, 8) | (NarrowValueMode::SignExtend64, 8) => {
+                ExtendOp::SXTB
+            }
+            (NarrowValueMode::ZeroExtend32, 8) | (NarrowValueMode::ZeroExtend64, 8) => {
+                ExtendOp::UXTB
+            }
+            (NarrowValueMode::SignExtend32, 16) | (NarrowValueMode::SignExtend64, 16) => {
+                ExtendOp::SXTH
+            }
+            (NarrowValueMode::ZeroExtend32, 16) | (NarrowValueMode::ZeroExtend64, 16) => {
+                ExtendOp::UXTH
+            }
+            (NarrowValueMode::SignExtend64, 32) => ExtendOp::SXTW,
+            (NarrowValueMode::ZeroExtend64, 32) => ExtendOp::UXTW,
+            _ => unreachable!(),
+        };
+        return Some((val, extendop));
+    }
+    None
+}
+
 pub(crate) fn put_input_in_rse_imm12<C: LowerCtx<I = Inst>>(
     ctx: &mut C,
     input: InsnInput,
@@ -470,35 +488,6 @@ pub(crate) fn put_input_in_rse_imm12<C: LowerCtx<I = Inst>>(
     ResultRSEImm12::from_rse(put_input_in_rse(ctx, input, narrow_mode))
 }
 
-/// Like `put_input_in_rse_imm12` above, except is allowed to negate the
-/// argument (assuming a two's-complement representation with the given bit
-/// width) if this allows use of 12-bit immediate. Used to flip `add`s with
-/// negative immediates to `sub`s (and vice-versa).
-pub(crate) fn put_input_in_rse_imm12_maybe_negated<C: LowerCtx<I = Inst>>(
-    ctx: &mut C,
-    input: InsnInput,
-    twos_complement_bits: usize,
-    narrow_mode: NarrowValueMode,
-) -> (ResultRSEImm12, bool) {
-    assert!(twos_complement_bits <= 64);
-    if let Some(imm_value) = input_to_const(ctx, input) {
-        if let Some(i) = Imm12::maybe_from_u64(imm_value) {
-            return (ResultRSEImm12::Imm12(i), false);
-        }
-        let sign_extended =
-            ((imm_value as i64) << (64 - twos_complement_bits)) >> (64 - twos_complement_bits);
-        let inverted = sign_extended.wrapping_neg();
-        if let Some(i) = Imm12::maybe_from_u64(inverted as u64) {
-            return (ResultRSEImm12::Imm12(i), true);
-        }
-    }
-
-    (
-        ResultRSEImm12::from_rse(put_input_in_rse(ctx, input, narrow_mode)),
-        false,
-    )
-}
-
 pub(crate) fn put_input_in_rs_immlogic<C: LowerCtx<I = Inst>>(
     ctx: &mut C,
     input: InsnInput,
@@ -1296,153 +1285,6 @@ pub(crate) fn maybe_input_insn_via_conv<C: LowerCtx<I = Inst>>(
     None
 }
 
-/// Pattern match an extending vector multiplication.
-/// Returns a tuple of the opcode to use, the two input registers and whether
-/// it's the 'high half' version of the instruction.
-pub(crate) fn match_vec_long_mul<C: LowerCtx<I = Inst>>(
-    c: &mut C,
-    insn: IRInst,
-    ext_op: Opcode,
-) -> Option<(VecRRRLongOp, regalloc::Reg, regalloc::Reg, bool)> {
-    let inputs = insn_inputs(c, insn);
-    if let Some(lhs) = maybe_input_insn(c, inputs[0], ext_op) {
-        if let Some(rhs) = maybe_input_insn(c, inputs[1], ext_op) {
-            let lhs_input = insn_inputs(c, lhs)[0];
-            let rhs_input = insn_inputs(c, rhs)[0];
-            let rn = put_input_in_reg(c, lhs_input, NarrowValueMode::None);
-            let rm = put_input_in_reg(c, rhs_input, NarrowValueMode::None);
-            let lane_type = c.output_ty(insn, 0).lane_type();
-            match (lane_type, ext_op) {
-                (I16, Opcode::SwidenLow) => return Some((VecRRRLongOp::Smull8, rn, rm, false)),
-                (I16, Opcode::SwidenHigh) => return Some((VecRRRLongOp::Smull8, rn, rm, true)),
-                (I16, Opcode::UwidenLow) => return Some((VecRRRLongOp::Umull8, rn, rm, false)),
-                (I16, Opcode::UwidenHigh) => return Some((VecRRRLongOp::Umull8, rn, rm, true)),
-                (I32, Opcode::SwidenLow) => return Some((VecRRRLongOp::Smull16, rn, rm, false)),
-                (I32, Opcode::SwidenHigh) => return Some((VecRRRLongOp::Smull16, rn, rm, true)),
-                (I32, Opcode::UwidenLow) => return Some((VecRRRLongOp::Umull16, rn, rm, false)),
-                (I32, Opcode::UwidenHigh) => return Some((VecRRRLongOp::Umull16, rn, rm, true)),
-                (I64, Opcode::SwidenLow) => return Some((VecRRRLongOp::Smull32, rn, rm, false)),
-                (I64, Opcode::SwidenHigh) => return Some((VecRRRLongOp::Smull32, rn, rm, true)),
-                (I64, Opcode::UwidenLow) => return Some((VecRRRLongOp::Umull32, rn, rm, false)),
-                (I64, Opcode::UwidenHigh) => return Some((VecRRRLongOp::Umull32, rn, rm, true)),
-                _ => {}
-            };
-        }
-    }
-    None
-}
-
-pub(crate) fn lower_i64x2_mul<C: LowerCtx<I = Inst>>(c: &mut C, insn: IRInst) {
-    let inputs = insn_inputs(c, insn);
-    let outputs = insn_outputs(c, insn);
-    let rd = get_output_reg(c, outputs[0]).regs()[0];
-    let rn = put_input_in_regs(c, inputs[0]).regs()[0];
-    let rm = put_input_in_regs(c, inputs[1]).regs()[0];
-
-    let tmp1 = c.alloc_tmp(I64X2).only_reg().unwrap();
-    let tmp2 = c.alloc_tmp(I64X2).only_reg().unwrap();
-
-    // This I64X2 multiplication is performed with several 32-bit
-    // operations.
-
-    // 64-bit numbers x and y, can be represented as:
-    //   x = a + 2^32(b)
-    //   y = c + 2^32(d)
-
-    // A 64-bit multiplication is:
-    //   x * y = ac + 2^32(ad + bc) + 2^64(bd)
-    // note: `2^64(bd)` can be ignored, the value is too large to fit in
-    // 64 bits.
-
-    // This sequence implements a I64X2 multiply, where the registers
-    // `rn` and `rm` are split up into 32-bit components:
-    //   rn = |d|c|b|a|
-    //   rm = |h|g|f|e|
-    //
-    //   rn * rm = |cg + 2^32(ch + dg)|ae + 2^32(af + be)|
-    //
-    //  The sequence is:
-    //  rev64 rd.4s, rm.4s
-    //  mul rd.4s, rd.4s, rn.4s
-    //  xtn tmp1.2s, rn.2d
-    //  addp rd.4s, rd.4s, rd.4s
-    //  xtn tmp2.2s, rm.2d
-    //  shll rd.2d, rd.2s, #32
-    //  umlal rd.2d, tmp2.2s, tmp1.2s
-
-    // Reverse the 32-bit elements in the 64-bit words.
-    //   rd = |g|h|e|f|
-    c.emit(Inst::VecMisc {
-        op: VecMisc2::Rev64,
-        rd,
-        rn: rm,
-        size: VectorSize::Size32x4,
-    });
-
-    // Calculate the high half components.
-    //   rd = |dg|ch|be|af|
-    //
-    // Note that this 32-bit multiply of the high half
-    // discards the bits that would overflow, same as
-    // if 64-bit operations were used. Also the Shll
-    // below would shift out the overflow bits anyway.
-    c.emit(Inst::VecRRR {
-        alu_op: VecALUOp::Mul,
-        rd,
-        rn: rd.to_reg(),
-        rm: rn,
-        size: VectorSize::Size32x4,
-    });
-
-    // Extract the low half components of rn.
-    //   tmp1 = |c|a|
-    c.emit(Inst::VecRRNarrow {
-        op: VecRRNarrowOp::Xtn64,
-        rd: tmp1,
-        rn,
-        high_half: false,
-    });
-
-    // Sum the respective high half components.
-    //   rd = |dg+ch|be+af||dg+ch|be+af|
-    c.emit(Inst::VecRRR {
-        alu_op: VecALUOp::Addp,
-        rd,
-        rn: rd.to_reg(),
-        rm: rd.to_reg(),
-        size: VectorSize::Size32x4,
-    });
-
-    // Extract the low half components of rm.
-    //   tmp2 = |g|e|
-    c.emit(Inst::VecRRNarrow {
-        op: VecRRNarrowOp::Xtn64,
-        rd: tmp2,
-        rn: rm,
-        high_half: false,
-    });
-
-    // Shift the high half components, into the high half.
-    //   rd = |dg+ch << 32|be+af << 32|
-    c.emit(Inst::VecRRLong {
-        op: VecRRLongOp::Shll32,
-        rd,
-        rn: rd.to_reg(),
-        high_half: false,
-    });
-
-    // Multiply the low components together, and accumulate with the high
-    // half.
-    //   rd = |rd[1] + cg|rd[0] + ae|
-    c.emit(Inst::VecRRRLong {
-        alu_op: VecRRRLongOp::Umlal32,
-        rd,
-        rn: tmp2.to_reg(),
-        rm: tmp1.to_reg(),
-        high_half: false,
-    });
-}
-
 /// Specifies what [lower_icmp] should do when lowering
 #[derive(Debug, Clone, PartialEq)]
 pub(crate) enum IcmpOutput {
